{       TBnVertScrollText version 1.00
        author: Benoe (wisnu_aribowo@hotmail.com)
        -- 2001 --

        Documentation : TBnVertScrollText.txt               }

unit BnVertScrollText;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Extctrls, Comctrls;

type
  TAlign = (alLeft,alCenter,alRight);
  TBnVertScrollText = class(TGraphicControl)
  private
    flines:tstringlist;
    fbgbmp:tbitmap;
    finterval:integer;
    frepeat,fenable,fstretch:boolean;
    falign:TAlign;

    linepos,posy,posx:integer;
    textbitmap,bgbitmap,copybitmap:tbitmap;
    sptimer:ttimer;
    procedure setlines(value:tstringlist);
    procedure setinterval(value:integer);
    procedure setbgbmp(value:tbitmap);
    procedure setenable(value:boolean);
    procedure setalign(value:talign);
    procedure setstretch(value:boolean);
    procedure inv;
    procedure ontimer(sender:tobject);
    procedure CMFontChanged(var Msg: TMessage); message CM_FontChanged;
  protected
    procedure loaded; override;
    procedure paint; override;
  public
    constructor create(aowner:tcomponent); override;
    destructor destroy; override;
    procedure resize; override;
  published
    property Lines:tstringlist read flines write setlines;
    property Interval:integer read finterval write setinterval;
    property BkgrndBitmap:tbitmap read fbgbmp write setbgbmp;
    property Font;
    property ParentFont;
    property RepeatScroll:boolean read frepeat write frepeat;
    property Enabled:boolean read fenable write setenable;
    property Alignment:TAlign read falign write setalign;
    property Stretch:boolean read fstretch write setstretch;
  end;

procedure Register;

implementation

constructor TBnVertScrollText.create(aowner:tcomponent);
begin
  inherited;

  textbitmap:=tbitmap.create;
  textbitmap.transparent:=true;
  bgbitmap:=tbitmap.create;
  copybitmap:=tbitmap.create;
  fbgbmp:=tbitmap.create;
  flines:=tstringlist.Create;
  flines.Add('Scrolling text goes here ....');
  finterval:=50; fenable:=true; frepeat:=true;
  falign:=alleft; fstretch:=true;

  width:=200; height:=100;
  posy:=height;

  sptimer:=ttimer.create(nil);
  sptimer.interval:=finterval;
  sptimer.ontimer:=ontimer;
end;

destructor TBnVertScrollText.destroy;
begin
  sptimer.free; fbgbmp.free; textbitmap.free;
  copybitmap.free; bgbitmap.free; flines.free;
  inherited;
end;

procedure TBnVertScrollText.loaded;
begin
  inv;
end;

procedure TBnVertScrollText.CMFontChanged(var Msg: TMessage);
begin
  inherited; inv;
  Msg.Result := 1;
end;

procedure TBnVertScrollText.setlines(value:tstringlist);
begin
  if flines<> value then begin
    flines.Assign(value);
    inv;
  end;
end;

procedure TBnVertScrollText.setinterval(value:integer);
begin
  if finterval<>value then begin
    finterval:=value; sptimer.interval:=value;
  end;
end;

procedure TBnVertScrollText.setbgbmp(value:tbitmap);
begin
  with fbgbmp do begin
    FreeImage;
    width:=value.Width; height:=value.height;
    canvas.draw(0,0,value);
  end;
  inv;
end;

procedure TBnVertScrollText.setenable(value:boolean);
begin
  fenable:=value;
  if fenable then begin
    posy:=height;
    sptimer.enabled:=true;
  end else begin
    sptimer.enabled:=false;
  end;
end;

procedure TBnVertScrollText.setalign(value:talign);
begin
  falign:=value; inv;
end;

procedure TBnVertScrollText.setstretch(value:boolean);
begin
  fstretch:=value; inv;
end;

procedure TBnVertScrollText.ontimer(sender:tobject);
begin
  if posy+linepos<0 then begin
    if frepeat then posy:=height else sptimer.enabled:=false;
  end;
  dec(posy); paint;
end;

procedure TBnVertScrollText.inv;
var aa:smallint;
begin
  with textbitmap do begin
    Canvas.font:=self.font;
    width:=self.Width;
    height:=canvas.textheight('text')*flines.Count;
    if self.font.color=clblue then begin
      Canvas.brush.color:=clgreen;
      transparentcolor:=clgreen;
    end else begin
      canvas.brush.color:=clblue;
      transparentcolor:=clblue;
    end;
    canvas.fillrect(rect(0,0,width,height));
    linepos:=0;
    for aa:=1 to flines.count do begin
      if falign=alleft then posx:=0;
      if falign=alright then posx:=width-canvas.textwidth(flines[aa-1]);
      if falign=alcenter then posx:=round((width-canvas.textwidth(flines[aa-1]))/2);
      canvas.TextOut(posx,linepos,flines[aa-1]);
      linepos:=linepos+canvas.textheight('text');
    end;
  end;
  with bgbitmap do begin
    width:=self.width; height:=self.height;
    freeimage;
    Canvas.FillRect(rect(0,0,width,height));
    if fstretch then canvas.stretchdraw(rect(0,0,width,height),fbgbmp)
    else canvas.draw(0,0,fbgbmp);
  end;
  with copybitmap do begin
    width:=self.width; height:=self.height;
  end;
  if parent<>nil then paint;
end;

procedure TBnVertScrollText.paint;
begin
  inherited;
  bitblt(copybitmap.canvas.handle,0,0,width,height,bgbitmap.canvas.handle,0,0,srccopy);
  copybitmap.canvas.draw(0,posy,textbitmap);
  bitblt(canvas.handle,0,0,width,height,copybitmap.canvas.handle,0,0,srccopy);
end;

procedure TBnVertScrollText.resize;
begin
  inherited; inv;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TBnVertScrollText]);
end;

end.
